#include <kos.h>
#include <dc/maple/keyboard.h>


extern uint8 romdisk[];
KOS_INIT_ROMDISK(romdisk);
char charrom[2048];
char keyboard;
int x=0,y=6;   //screen co-ords
/*
The routine below is character rom generator "emulator", the routine
is slow and unoptimised.. by since it's text it doesn't really matter.

How it works is that most 80's computers character roms (including the PC)
had the chars stored as 8x8 pixel images 8bits by 8bits so each charater
is 8 bytes in size in the rom they organise  starting
from the top line towards the bottom. example A

 Bits             Mem Offset
...11...              0
..1111..              1
11....11              2
11....11              3
11111111              4
11111111              5
11....11              6
11....11              7
11....11              8

On some roms they actually organised in the standard ASCII
order so to get a start address of the charates data
simply calc ASCIIValue*8, on the PC Char rom this is
true, but for most other roms you'll need to make
a translation table or re-orient the rom itself.

since the data is in "bits" to represent a pixel we
litterly need to convert bits to words/dwords to draw
on the screen.

So simply get the first byte of the character then roll
bitmask to check each bit for off and on and draw a pixel
on the screen accordingly. as I did below.
You could always hold the bitmask 0x128 and shift the char data
to chardata<<1 to achieve the same effect.
The rest is a matter of proper screen alignment.

A simpler way to do this is litterly make a bitmap image in
your screen color depth of all your characters predrawn
not as they were to appear on the screen, but in the same
format as we just saw, but instead of bits you use words(16bpp)
or dwords(32bpp) and then its only a simple matter of linearly
copying the data. 
In emulators you would do a simmilar approach at runtime
by converting  the char rom into that format, so you could 
litterally gain 300%-500% speed increase.
Of course there's the happy middle ground.. which is litterally
a tile engine draw your character on a bitmap, and copy them to
the screen as you would tiles.. or sprites.

The font included is actually a modifed Commodore +4 font, I've re-ordered it
so that it actually matches ASCII order

you could also dump a PC Rom font.. the first 1/2 of the chars
(standard ASCII) are located at 0xffa6e (1KB), the 2nd half (extended ascii)
can be found from a _far pointer a 0x0007c, of course you'll have
to grab it in real DOS, with debug, or a simple bit of code.

Also as the routine is now I say 80x50 40x25 etc..
it's actuall 80x60 or 40x30 (480/8 or 16) but you'll
have to change modifiy the top/bottom left/right
of the screen since the overscan on most TV's will
cause some text to be off screen.

Just as a note if you use 640x480 VGA (which also works on NTSC)
you'll loose part of the bottom of the screen. If you use the 
ntsc (which also works on VGA) you'll loose the top of the screen
to overscan on most TV's so always check, and try to keep
a safe margin from the edge of the screen. On magenetic
I kept about 4-6 chars (40x30) from either side, and about
8 chars from the bottom..

*/

///////////////////////////////////////////////////////////////////////////

void dcprintf(uint16 *address,int scrnwidth,int reverse, unsigned char character,uint16 color){
     uint8 i,j,k=0;
     uint16 *screenptr;
     uint8 bitmask;
     int c;


     for(i=0;i<16;i+=2){
        bitmask=0x80;

        for(j=0;j<8;j++){
            c=charrom[k+((character)*8)]&(bitmask); //get char 1 bit at a time
            if(reverse!=0)c=!c;  //negate the bit so the char is reversed
            bitmask = bitmask >>1; //prepare to get next bit
            if(c!=0){
            screenptr=address+((i<<9)+(i<<7));
            screenptr[j]= color;      //draw the bit from the char as a pixel on screen
            //screenptr[j+1]= color; //if you want 40x25

            screenptr=address+((i<<9)+(i<<7))+640; //remove section if you want 80x50 or 40x50
            screenptr[j]= color;
            //screenptr[j+1]= color; //if you want 40x25

           }
            else{
            screenptr=address+((i<<9)+(i<<7));
            screenptr[j]= 0x0000;
            //screenptr[j+1]= 0x0000; //if you want 40x25

            screenptr=address+((i<<9)+(i<<7))+640; //remove section if you want 80x50 or 40x50
            screenptr[j]= 0x0000;
            //screenptr[j+1]= 0x0000; //if you want 40x25
            }
            }//end for J
            k++; //move pointer to next (byte) or line of bits in the character
            }//end for I
        }//end dcprintf
///////////////////////////////////////////////////////////////////////////
//this routine needs to be replaced with one that actually
//take into acount the end of the screen etc..

void sdcprintf(uint16 *address,int scrnwidth,int reverse, char * character,uint16 color){

     while(*character){
          dcprintf(address,scrnwidth,reverse, *character,color);
          character++;
          address+=8; //modify me if you change the character width
     }
}
///////////////////////////////////////////////////////////////////////////
char getch(void){
        char c;
        c=-1;
        while(c==-1){ //if there isn't any key press loop
        if(keyboard!=0)c=kbd_get_key();

        }
        return c;
        }
///////////////////////////////////////////////////////////////////////////
void parse_command(char *userinput){
      sdcprintf(vram_s+(y*16*640)+x*8,640,0,"I dont know how to ",0xffff);
      x+=19;
      sdcprintf(vram_s+(y*16*640)+x*8,640,0,userinput,0xffff);
      y++;
      x=0;
}
///////////////////////////////////////////////////////////////////////////

void main(void){
      char done=0;
      short ch;  //used to get a char
      char textbuffer[32];
      char counter=0;
      FILE *fh;

      //load the font
      if (!(fh=fopen("/rd/font.bin","rb")));
      fread(charrom,1,1400,fh);
      fclose(fh);

      keyboard = maple_first_kb();  //check for a keyboard if yes nonzero
      if(keyboard!=0){
                    kbd_set_queue(1); //setup the keyboard
                    while(ch!=-1)ch=kbd_get_key();//empty the kb buffer in case
      }

      vid_set_mode(3, 0); //640x480 1:5:5:5

      sdcprintf(vram_s+(2*16*640)+2*8,640,0,"Welcome To Textalot...",0x0200);
      sdcprintf(vram_s+(4*16*640)+0*8,640,0,"You are in a Forest with a CRAZY Clown..",0xffff);
      sdcprintf(vram_s+(5*16*640)+0*8,640,0,"What do you want to do?",0xffff);
      //This is setup as vram_s =0xa5000000 point to the start of the screen
      //y*16*640  ypos * character height(according to dcprinf)
      //+x*8  + xpos * character width (according to dcprinf)
      //640 = screen width 
      //0 =Normal 1 = Reverse (AKA inverse)
      //Text
      //And the color (RGB5:5:5) White=0x7FFF Blue=0x001F Red=0x07C00 Green = 0x03e0


      while(!done){
      dcprintf(vram_s+(y*16*640)+x*8,640,1,' ',0x4000);//make a nice red cursor
               ch=getch();
               if(ch==0x1b)done=1;
      dcprintf(vram_s+(y*16*640)+x*8,640,0,' ',0x0000);//erase the cursor

      if(ch==0x0d){ //hit return
                 x=0;// zero the x
                 y++;// goto the next line
                 textbuffer[counter]=0x00;
                 counter=0;
                 parse_command(textbuffer);
                 }

      if(ch==0x08 && x>0 && counter>0){ //hit delete
                 x--;// back up the x
                 dcprintf(vram_s+(y*16*640)+x*8,640,0,' ',0x0000); //erase the character
                 counter--;
                 textbuffer[counter]=0x00;

                 }
      if(ch>=32 && ch<=122 ){//check if it's a printable character
                 dcprintf(vram_s+(y*16*640)+x*8,640,0,ch,0xffff);
                 x++;
                 textbuffer[counter]=ch; //just for fun
                 counter++;

                }//ascii check


       if(x>40){ //reached the end of a line
                 x=0; //position back to start
                 y++; //one line down
               }//end x check


       if(y>25){//reached the end of the sreen (this should actually be a CLRSCRN routine)
                  vid_empty(); //clear the screen start over
                  x=0;         //here you'd really want a scroll routine
                  y=0;         //that copies everything on the screen
               } //end y check //up by 1 char (8 or 16 lines), then clears the only the
                               //bottom line places y=25 and x=0;


      }//end done

}
